@@ -98,6 +98,8 @@ void renumber_xscreens(nvLayoutPtr layout);
 void screen_unlink_display(nvDisplayPtr display);
 void screen_link_display(nvScreenPtr screen, nvDisplayPtr display);
 void screen_remove_display(nvDisplayPtr display);
+void screen_unlink_prime_display(nvPrimeDisplayPtr prime);
+void screen_link_prime_display(nvScreenPtr screen, nvPrimeDisplayPtr prime);
 gchar * screen_get_metamode_str(nvScreenPtr screen, int metamode_idx,
                                 int force_target_id_name);
 void link_screen_to_gpu(nvScreenPtr screen, nvGpuPtr gpu);
