@@ -51,9 +51,7 @@
 #include "ctkmultisample.h"
 #include "ctkthermal.h"
 #include "ctkpowermizer.h"
-#include "ctkclocks.h"
 #include "ctkvcs.h"
-#include "ctkpowersavings.h"
 #include "ctk3dvisionpro.h"
 
 #include "ctkdisplaydevice.h"
@@ -61,6 +59,7 @@
 #include "ctkdisplayconfig.h"
 #include "ctkserver.h"
 #include "ctkecc.h"
+#include "ctkgridlicense.h"
 #include "ctkvdpau.h"
 
 #include "ctkappprofile.h"
@@ -89,13 +88,13 @@ enum {
 
 typedef struct {
     CtkWindow *window;
-    CtkEvent *event;
-    NvCtrlAttributeHandle *gpu_handle;
+    CtrlTarget *gpu_target;
     GtkTextTagTable *tag_table;
 
     GtkTreeIter parent_iter;
 
     GtkTreeIter *display_iters;
+    CtkEvent **display_events;
     int num_displays;
 
 } UpdateDisplaysData;
@@ -107,7 +106,11 @@ typedef void (*unselect_widget_func_t)(GtkWidget *);
 
 static void ctk_window_class_init(CtkWindowClass *);
 
+#ifdef CTK_GTK3
+static void ctk_window_real_destroy(GtkWidget *);
+#else
 static void ctk_window_real_destroy(GtkObject *);
+#endif
 
 static void add_page(GtkWidget *, GtkTextBuffer *, CtkWindow *,
                      GtkTreeIter *, GtkTreeIter *, const gchar *,
@@ -121,18 +124,15 @@ static void quit_response(GtkWidget *, gint, gpointer);
 static void save_settings_and_exit(CtkWindow *);
 
 static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
-                                NvCtrlAttributeHandle *handle,
+                                CtrlTarget *gpu_target,
                                 CtkEvent *ctk_event,
                                 GtkTextTagTable *tag_table,
                                 UpdateDisplaysData *data,
                                 ParsedAttribute *p);
 
-static void update_display_devices(GtkObject *object, gpointer arg1,
+static void update_display_devices(GtkWidget *object, CtrlEvent *event,
                                    gpointer user_data);
 
-static void ctk_window_destroyed(GtkObject *object, gpointer arg1,
-                                 gpointer user_data);
-
 
 static GObjectClass *parent_class;
 
@@ -178,6 +178,15 @@ GType ctk_window_get_type(void)
 
 static void ctk_window_class_init(CtkWindowClass *ctk_window_class)
 {
+#ifdef CTK_GTK3
+    GtkWidgetClass *widget_class;
+
+    widget_class = (GtkWidgetClass *) ctk_window_class;
+
+    parent_class = g_type_class_peek_parent(ctk_window_class);
+
+    widget_class->destroy = ctk_window_real_destroy;
+#else
     GtkObjectClass *object_class;
 
     object_class = (GtkObjectClass *) ctk_window_class;
@@ -185,7 +194,7 @@ static void ctk_window_class_init(CtkWindowClass *ctk_window_class)
     parent_class = g_type_class_peek_parent(ctk_window_class);
 
     object_class->destroy = ctk_window_real_destroy;
-
+#endif
 } /* ctk_window_class_init() */
 
 
@@ -195,25 +204,41 @@ static void ctk_window_class_init(CtkWindowClass *ctk_window_class)
  * the configuration file here?
  */
 
+#ifdef CTK_GTK3
+static void ctk_window_real_destroy(GtkWidget *object)
+{
+    GTK_WIDGET_CLASS(parent_class)->destroy(object);
+    gtk_main_quit();
+
+} /* ctk_window_real_destroy() */
+#else
 static void ctk_window_real_destroy(GtkObject *object)
 {
     GTK_OBJECT_CLASS(parent_class)->destroy(object);
     gtk_main_quit();
 
 } /* ctk_window_real_destroy() */
+#endif
 
 
 
-/* 
- * ctk_window_destroyed() - called when ctk_window destroyed.
+/*
+ * confirm_quit_and_save() - Shows the user the quit dialog and
+ * saves configuration information before exiting.
  */
 
-static void ctk_window_destroyed(GtkObject *object, gpointer arg1,
-                                 gpointer user_data)
+static void confirm_quit_and_save(CtkWindow *ctk_window)
 {
-    CtkWindow *ctk_window = CTK_WINDOW(object);
-    save_settings_and_exit(ctk_window);
-} /* ctk_window_destroyed() */
+    CtkConfig *ctk_config = ctk_window->ctk_config;
+
+    if (ctk_config->conf->booleans & CONFIG_PROPERTIES_SHOW_QUIT_DIALOG) {
+        /* ask for confirmation */
+        gtk_widget_show_all(ctk_window->quit_dialog);
+    } else {
+        /* doesn't return */
+        save_settings_and_exit(ctk_window);
+    }
+}
 
 
 
@@ -224,16 +249,24 @@ static void ctk_window_destroyed(GtkObject *object, gpointer arg1,
 static void close_button_clicked(GtkButton *button, gpointer user_data)
 {
     CtkWindow *ctk_window = CTK_WINDOW(user_data);
-    CtkConfig *ctk_config = ctk_window->ctk_config;
 
-    if (ctk_config->conf->booleans & CONFIG_PROPERTIES_SHOW_QUIT_DIALOG) {
-        /* ask for confirmation */
-        gtk_widget_show_all(ctk_window->quit_dialog);
-    } else {
-        /* doesn't return */
-        save_settings_and_exit(ctk_window);
-    }
-} /* close_button_clicked() */
+    confirm_quit_and_save(ctk_window);
+}
+
+
+
+static gboolean ctk_window_delete_event(GObject *object)
+{
+    CtkWindow *ctk_window = CTK_WINDOW(object);
+
+    confirm_quit_and_save(ctk_window);
+
+    /* gtk_main_quit() will be called above if the user really wants to quit,
+     * so halt the progress of the delete-event here so we don't exit
+     * prematurely.
+     */
+    return TRUE;
+}
 
 
 
@@ -260,7 +293,7 @@ static void help_button_toggled(GtkToggleButton *button, gpointer user_data)
         }
         gtk_widget_show_all(ctk_window->ctk_help);
     } else {
-        gtk_widget_hide_all(ctk_window->ctk_help);
+        gtk_widget_hide(ctk_window->ctk_help);
     }
 
 } /* help_button_toggled() */
@@ -364,12 +397,13 @@ static void row_activated_event(GtkTreeView        *view,
 
 
 
-static gboolean has_randr_gamma(NvCtrlAttributeHandle *handle)
+static gboolean has_randr_gamma(CtrlTarget *target)
 {
     ReturnStatus ret;
     int val;
 
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE, &val);
+    ret = NvCtrlGetAttribute(target, NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE,
+                             &val);
 
     return ((ret == NvCtrlSuccess) && (val == 1));
 }
@@ -379,7 +413,7 @@ static gboolean has_randr_gamma(NvCtrlAttributeHandle *handle)
  */
 
 GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
-                          CtrlHandles *h)
+                          CtrlSystem *system)
 {
     GObject *object;
     CtkWindow *ctk_window;
@@ -404,13 +438,16 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     GtkTextTagTable *tag_table;
 
     GtkTextBuffer *help;
-        
+
+    CtrlTargetNode *node;
+    CtrlTarget *server_target = NULL;
+
     CtkEvent *ctk_event;
     CtkConfig *ctk_config;
-    
-    gint column_offset, i;
+
+    gint column_offset;
     gboolean slimm_page_added; /* XXX Kludge to only show one SLIMM page */
-    
+
     /* create the new object */
 
     object = g_object_new(CTK_TYPE_WINDOW, NULL);
@@ -422,7 +459,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     
     /* create the config object */
 
-    ctk_window->ctk_config = CTK_CONFIG(ctk_config_new(conf, h));
+    ctk_window->ctk_config = CTK_CONFIG(ctk_config_new(conf, system));
     ctk_config = ctk_window->ctk_config;
     
     /* create the quit dialog */
@@ -527,7 +564,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     /* Added row activated event to the tree view */
 
     g_signal_connect(ctk_window->treeview, "row_activated",
-                     G_CALLBACK(row_activated_event), GTK_OBJECT(ctk_window));
+                     G_CALLBACK(row_activated_event), GTK_WIDGET(ctk_window));
 
     selection = gtk_tree_view_get_selection(ctk_window->treeview);
 
@@ -562,29 +599,22 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
     /* X Server info & configuration */
 
-    if (h->targets[X_SCREEN_TARGET].n) {
+    if (system->targets[X_SCREEN_TARGET]) {
 
-        NvCtrlAttributeHandle *screen_handle = NULL;
         GtkWidget *child;
-        int i;
 
         /*
          * XXX For now, just use the first handle in the list
          *     to communicate with the X server for these two
-         *     pages.
+         *     pages and the app profile page below.
          */
 
-        for (i = 0 ; i < h->targets[X_SCREEN_TARGET].n; i++) {
-            if (h->targets[X_SCREEN_TARGET].t[i].h) {
-                screen_handle = h->targets[X_SCREEN_TARGET].t[i].h;
-                break;
-            }
-        }
-        if (screen_handle) {
+        server_target = NvCtrlGetDefaultTargetByType(system, X_SCREEN_TARGET);
+        if (server_target) {
 
             /* X Server information */
-            
-            child = ctk_server_new(screen_handle, ctk_config);
+
+            child = ctk_server_new(server_target, ctk_config);
             add_page(child,
                      ctk_server_create_help(tag_table,
                                             CTK_SERVER(child)),
@@ -593,7 +623,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
             /* X Server Display Configuration */
 
-            child = ctk_display_config_new(screen_handle, ctk_config);
+            child = ctk_display_config_new(server_target, ctk_config);
             if (child) {
                 add_page(child,
                          ctk_display_config_create_help(tag_table,
@@ -610,20 +640,22 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     /* add the per-screen entries into the tree model */
 
     slimm_page_added = FALSE;
-    for (i = 0; i < h->targets[X_SCREEN_TARGET].n; i++) {
+    for (node = system->targets[X_SCREEN_TARGET]; node; node = node->next) {
 
         gchar *screen_name;
         GtkWidget *child;
-        NvCtrlAttributeHandle *screen_handle = h->targets[X_SCREEN_TARGET].t[i].h;
+        CtrlTarget *screen_target = node->t;
 
-        if (!screen_handle) continue;
+        if (screen_target == NULL || screen_target->h == NULL) {
+            continue;
+        }
 
         /* create the object for receiving NV-CONTROL events */
 
-        ctk_event = CTK_EVENT(ctk_event_new(screen_handle));
-        
+        ctk_event = CTK_EVENT(ctk_event_new(screen_target));
+
         screen_name = g_strdup_printf("X Screen %d",
-                                      NvCtrlGetTargetId(screen_handle));
+                                      NvCtrlGetTargetId(screen_target));
 
         /* create the screen entry */
 
@@ -633,10 +665,10 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
         /* Screen information */
 
-        screen_name = NvCtrlGetDisplayName(screen_handle);
+        screen_name = NvCtrlGetDisplayName(screen_target);
 
-        child = ctk_screen_new(screen_handle, ctk_event);
-        gtk_object_ref(GTK_OBJECT(child));
+        child = ctk_screen_new(screen_target, ctk_event);
+        g_object_ref(G_OBJECT(child));
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
@@ -651,7 +683,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         if (!slimm_page_added) {
             /* SLI Mosaic Mode information */
 
-            child = ctk_slimm_new(screen_handle, ctk_event, ctk_config);
+            child = ctk_slimm_new(screen_target, ctk_event, ctk_config);
             if (child) {
                 slimm_page_added = TRUE;
                 help = ctk_slimm_create_help(tag_table, "SLI Mosaic Mode Settings");
@@ -665,9 +697,9 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
          * available
          */
 
-        if (!has_randr_gamma(screen_handle)) {
+        if (!has_randr_gamma(screen_target)) {
 
-            child = ctk_color_correction_page_new(screen_handle, ctk_config,
+            child = ctk_color_correction_page_new(screen_target, ctk_config,
                                                   ctk_window->attribute_list,
                                                   ctk_event);
             if (child) {
@@ -679,7 +711,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
         /* xvideo settings  */
 
-        child = ctk_xvideo_new(screen_handle, ctk_config, ctk_event);
+        child = ctk_xvideo_new(screen_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_xvideo_create_help(tag_table, CTK_XVIDEO(child));
             add_page(child, help, ctk_window, &iter, NULL,
@@ -688,7 +720,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
         /* opengl settings */
 
-        child = ctk_opengl_new(screen_handle, ctk_config, ctk_event);
+        child = ctk_opengl_new(screen_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_opengl_create_help(tag_table, CTK_OPENGL(child));
             add_page(child, help, ctk_window, &iter, NULL, "OpenGL Settings",
@@ -698,7 +730,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
         /* GLX Information */
 
-        child = ctk_glx_new(screen_handle, ctk_config, ctk_event);
+        child = ctk_glx_new(screen_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_glx_create_help(tag_table, CTK_GLX(child));
             add_page(child, help, ctk_window, &iter, NULL,
@@ -708,7 +740,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
         /* multisample settings */
 
-        child = ctk_multisample_new(screen_handle, ctk_config, ctk_event);
+        child = ctk_multisample_new(screen_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_multisample_create_help(tag_table,
                                                CTK_MULTISAMPLE(child));
@@ -718,16 +750,16 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
 
         /* VDPAU Information */
-        child = ctk_vdpau_new(screen_handle, ctk_config, ctk_event);
+        child = ctk_vdpau_new(screen_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_vdpau_create_help(tag_table, CTK_VDPAU(child));
             add_page(child, help, ctk_window, &iter, NULL, "VDPAU Information",
                      NULL, NULL, NULL);
         }
-        
+
         /* gvo (Graphics To Video Out) */
-        
-        child = ctk_gvo_new(screen_handle, ctk_config, ctk_event);
+
+        child = ctk_gvo_new(screen_target, ctk_config, ctk_event);
         if (child) {
             GtkWidget *gvo_parent = child;
             GtkTreeIter child_iter;
@@ -738,7 +770,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
             /* GVO Sync options */
 
-            child = ctk_gvo_sync_new(screen_handle,  GTK_WIDGET(ctk_window),
+            child = ctk_gvo_sync_new(screen_target,  GTK_WIDGET(ctk_window),
                                      ctk_config, ctk_event,
                                      CTK_GVO(gvo_parent));
             if (child) {
@@ -751,7 +783,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
             /* GVO color space conversion */
 
-            child = ctk_gvo_csc_new(screen_handle, ctk_config, ctk_event,
+            child = ctk_gvo_csc_new(screen_target, ctk_config, ctk_event,
                                     CTK_GVO(gvo_parent));
             if (child) {
                 help = ctk_gvo_csc_create_help(tag_table, CTK_GVO_CSC(child));
@@ -764,35 +796,36 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
     /* add the per-gpu entries into the tree model */
 
-    for (i = 0; i < h->targets[GPU_TARGET].n; i++) {
-        
+    for (node = system->targets[GPU_TARGET]; node; node = node->next) {
+
         gchar *gpu_name;
         GtkWidget *child;
-        NvCtrlAttributeHandle *gpu_handle = h->targets[GPU_TARGET].t[i].h;
+        CtrlTarget *gpu_target = node->t;
         UpdateDisplaysData *data;
 
 
-        if (!gpu_handle) continue;
+        if (gpu_target == NULL || gpu_target->h == NULL) {
+            continue;
+        }
 
         /* create the gpu entry name */
 
-        gpu_name = create_gpu_name_string(gpu_handle);
+        gpu_name = create_gpu_name_string(gpu_target);
         if (!gpu_name) continue;
 
         /* create the object for receiving NV-CONTROL events */
 
-        ctk_event = CTK_EVENT(ctk_event_new(gpu_handle));
-       
+        ctk_event = CTK_EVENT(ctk_event_new(gpu_target));
+
         /* create the gpu entry */
 
         gtk_tree_store_append(ctk_window->tree_store, &iter, NULL);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_LABEL_COLUMN, gpu_name, -1);
 
-        child = ctk_gpu_new(gpu_handle, h->targets[X_SCREEN_TARGET].t, ctk_event,
-                            ctk_config);
+        child = ctk_gpu_new(gpu_target, ctk_event, ctk_config);
 
-        gtk_object_ref(GTK_OBJECT(child));
+        g_object_ref(G_OBJECT(child));
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
@@ -808,18 +841,9 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                            CTK_WINDOW_UNSELECT_WIDGET_FUNC_COLUMN,
                            ctk_gpu_page_unselect, -1);
 
-        /* power savings */
-
-        child = ctk_power_savings_new(gpu_handle, ctk_config, ctk_event);
-        if (child) {
-            help = ctk_power_savings_create_help(tag_table, CTK_POWER_SAVINGS(child));
-            add_page(child, help, ctk_window, &iter, NULL,
-                     "Power Savings Settings", NULL, NULL, NULL);
-        }
-
         /* thermal information */
 
-        child = ctk_thermal_new(gpu_handle, ctk_config, ctk_event);
+        child = ctk_thermal_new(gpu_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_thermal_create_help(tag_table, CTK_THERMAL(child));
             add_page(child, help, ctk_window, &iter, NULL, "Thermal Settings",
@@ -827,34 +851,26 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         }
 
         /* Powermizer information */
-        child = ctk_powermizer_new(gpu_handle, ctk_config, ctk_event);
+        child = ctk_powermizer_new(gpu_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_powermizer_create_help(tag_table, CTK_POWERMIZER(child));
             add_page(child, help, ctk_window, &iter, NULL, "PowerMizer",
                      NULL, ctk_powermizer_start_timer, 
                      ctk_powermizer_stop_timer);
         }
- 
-        /* clocks (GPU overclocking) */
 
-        child = ctk_clocks_new(gpu_handle, ctk_config, ctk_event);
-        if (child) {
-            help = ctk_clocks_create_help(tag_table, CTK_CLOCKS(child));
-            add_page(child, help, ctk_window, &iter, NULL, "Clock Frequencies",
-                     NULL, ctk_clocks_select, NULL);
-        }
         /* ECC Information */
-        child = ctk_ecc_new(gpu_handle, ctk_config, ctk_event);
+        child = ctk_ecc_new(gpu_target, ctk_config, ctk_event);
         if (child) {
             help = ctk_ecc_create_help(tag_table, CTK_ECC(child));
             add_page(child, help, ctk_window, &iter, NULL, "ECC Settings",
                      NULL, ctk_ecc_start_timer, ctk_ecc_stop_timer);
         }
+
         /* display devices */
-        data = calloc(1, sizeof(UpdateDisplaysData));
+        data = calloc(1, sizeof(*data));
         data->window = ctk_window;
-        data->event = ctk_event;
-        data->gpu_handle = gpu_handle;
+        data->gpu_target = gpu_target;
         data->parent_iter = iter;
         data->tag_table = tag_table;
 
@@ -868,49 +884,51 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                          G_CALLBACK(update_display_devices),
                          (gpointer) data);
 
-        add_display_devices(ctk_window, &iter, gpu_handle, ctk_event, tag_table,
+        add_display_devices(ctk_window, &iter, gpu_target, ctk_event, tag_table,
                             data, ctk_window->attribute_list);
     }
 
     /* add the per-vcs (e.g. Quadro Plex) entries into the tree model */
 
-    for (i = 0; i < h->targets[VCS_TARGET].n; i++) {
+    for (node = system->targets[VCS_TARGET]; node; node = node->next) {
 
         gchar *vcs_product_name;
         gchar *vcs_name;
         GtkWidget *child;
         ReturnStatus ret;
-        NvCtrlAttributeHandle *vcs_handle = h->targets[VCS_TARGET].t[i].h;
+        CtrlTarget *vcs_target = node->t;
 
-        if (!vcs_handle) continue;
+        if (vcs_target == NULL || vcs_target->h == NULL) {
+            continue;
+        }
 
         /* create the vcs entry name */
 
-        ret = NvCtrlGetStringAttribute(vcs_handle,
+        ret = NvCtrlGetStringAttribute(vcs_target,
                                        NV_CTRL_STRING_VCSC_PRODUCT_NAME,
                                        &vcs_product_name);
         if (ret == NvCtrlSuccess && vcs_product_name) {
             vcs_name = g_strdup_printf("VCS %d - (%s)",
-                                        NvCtrlGetTargetId(vcs_handle),
+                                        NvCtrlGetTargetId(vcs_target),
                                         vcs_product_name);
-            XFree(vcs_product_name);
+            free(vcs_product_name);
         } else {
             vcs_name =  g_strdup_printf("VCS %d - (Unknown)",
-                                        NvCtrlGetTargetId(vcs_handle));
+                                        NvCtrlGetTargetId(vcs_target));
         }
         if (!vcs_name) continue;
 
         /* create the object for receiving NV-CONTROL events */
 
-        ctk_event = CTK_EVENT(ctk_event_new(vcs_handle));
+        ctk_event = CTK_EVENT(ctk_event_new(vcs_target));
 
         /* create the vcs entry */
 
         gtk_tree_store_append(ctk_window->tree_store, &iter, NULL);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_LABEL_COLUMN, vcs_name, -1);
-        child = ctk_vcs_new(vcs_handle, ctk_config);
-        gtk_object_ref(GTK_OBJECT(child));
+        child = ctk_vcs_new(vcs_target, ctk_config);
+        g_object_ref(G_OBJECT(child));
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
@@ -930,36 +948,38 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
     /* add the per gvi entries into the tree model */
 
-    for (i = 0; i < h->targets[GVI_TARGET].n; i++) {
+    for (node = system->targets[GVI_TARGET]; node; node = node->next) {
 
         gchar *gvi_name;
         GtkWidget *child;
-        NvCtrlAttributeHandle *gvi_handle = h->targets[GVI_TARGET].t[i].h;
+        CtrlTarget *gvi_target = node->t;
 
-        if (!gvi_handle) continue;
+        if (gvi_target == NULL || gvi_target->h == NULL) {
+            continue;
+        }
 
         /* create the gvi entry name */
 
-        if (h->targets[GVI_TARGET].n > 1) {
+        if (node->next) {
             gvi_name = g_strdup_printf("Graphics to Video In %d",
-                                       NvCtrlGetTargetId(gvi_handle));
+                                       NvCtrlGetTargetId(gvi_target));
         } else {
             gvi_name =  g_strdup_printf("Graphics to Video In");
         }
-        
+
         if (!gvi_name) continue;
 
         /* create the object for receiving NV-CONTROL events */
 
-        ctk_event = CTK_EVENT(ctk_event_new(gvi_handle));
+        ctk_event = CTK_EVENT(ctk_event_new(gvi_target));
 
         /* create the gvi entry */
 
         gtk_tree_store_append(ctk_window->tree_store, &iter, NULL);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_LABEL_COLUMN, gvi_name, -1);
-        child = ctk_gvi_new(gvi_handle, ctk_config, ctk_event);
-        gtk_object_ref(GTK_OBJECT(child));
+        child = ctk_gvi_new(gvi_target, ctk_config, ctk_event);
+        g_object_ref(G_OBJECT(child));
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
@@ -981,13 +1001,15 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
      * frame lock
      */
 
-    for (i = 0; i < h->targets[X_SCREEN_TARGET].n; i++) {
+    for (node = system->targets[X_SCREEN_TARGET]; node; node = node->next) {
 
-        NvCtrlAttributeHandle *screen_handle = h->targets[X_SCREEN_TARGET].t[i].h;
+        CtrlTarget *screen_target = node->t;
 
-        if (!screen_handle) continue;
-        
-        widget = ctk_framelock_new(screen_handle, GTK_WIDGET(ctk_window),
+        if (screen_target == NULL || screen_target->h == NULL) {
+            continue;
+        }
+
+        widget = ctk_framelock_new(screen_target, GTK_WIDGET(ctk_window),
                                    ctk_config, ctk_window->attribute_list);
         if (!widget) continue;
 
@@ -1001,16 +1023,20 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
     /* add NVIDIA 3D VisionPro dongle configuration page */
 
-    for (i = 0; i < h->targets[NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET].n; i++) {
-        NvCtrlAttributeHandle *svp_handle = h->targets[NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET].t[i].h;
+    for (node = system->targets[NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET];
+         node;
+         node = node->next) {
+        CtrlTarget *svp_target = node->t;
 
-        if (!svp_handle) continue;
+        if (svp_target == NULL || svp_target->h == NULL) {
+            continue;
+        }
 
         /* create the object for receiving NV-CONTROL events */
 
-        ctk_event = CTK_EVENT(ctk_event_new(svp_handle));
+        ctk_event = CTK_EVENT(ctk_event_new(svp_target));
 
-        widget = ctk_3d_vision_pro_new(svp_handle, ctk_config,
+        widget = ctk_3d_vision_pro_new(svp_target, ctk_config,
                                        ctk_window->attribute_list,
                                        ctk_event);
         if (!widget) continue;
@@ -1022,11 +1048,25 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     }
 
     /* app profile configuration */
-    widget = ctk_app_profile_new(ctk_config);
+    widget = ctk_app_profile_new(server_target, ctk_config);
+    if (widget) {
+        add_page(widget, ctk_app_profile_create_help(CTK_APP_PROFILE(widget), tag_table),
+                 ctk_window, NULL, NULL, "Application Profiles",
+                 NULL, NULL, NULL);
+    }
 
-    add_page(widget, ctk_app_profile_create_help(CTK_APP_PROFILE(widget), tag_table),
-             ctk_window, NULL, NULL, "Application Profiles",
-             NULL, NULL, NULL);
+    /* Manage GRID License Information */
+    for (node = system->targets[GPU_TARGET]; node; node = node->next) {
+        widget = ctk_manage_grid_license_new(node->t, ctk_config);
+        if (widget) {
+            help = ctk_manage_grid_license_create_help(tag_table,
+                                                       CTK_MANAGE_GRID_LICENSE(widget));
+            add_page(widget, help, ctk_window, NULL, NULL, "Manage License",
+                     NULL, ctk_manage_grid_license_start_timer,
+                     ctk_manage_grid_license_stop_timer);
+            break; /* only add the page once */
+        }
+    }
 
     /* nvidia-settings configuration */
 
@@ -1049,7 +1089,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
      */
 
     g_signal_connect(selection, "changed", G_CALLBACK(tree_selection_changed),
-                     GTK_OBJECT(ctk_window));
+                     GTK_WIDGET(ctk_window));
 
     gtk_widget_show_all(GTK_WIDGET(ctk_window->treeview));
     gtk_tree_view_expand_all(ctk_window->treeview);
@@ -1075,17 +1115,18 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
      */
 
     label = gtk_label_new("XXXXXX Server Display ConfigurationXXXX");
-    gtk_widget_size_request(label, &req);
+    gtk_widget_show(label);
+    ctk_widget_get_preferred_size(label, &req);
     width = req.width;
     gtk_widget_destroy(label);
 
     /* Get the width of the tree view scroll window */
-    gtk_widget_size_request(sw, &req);
+    ctk_widget_get_preferred_size(sw, &req);
 
     /* If the scroll window is too wide, make it slimmer and
      * allow users to scroll horizontally (also allow resizing).
      */
-    if ( width < req.width ) {
+    if (width < req.width && width > 0) {
         gtk_widget_set_size_request(sw, width, -1);
         gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                        GTK_POLICY_AUTOMATIC,
@@ -1094,8 +1135,8 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
     /* Add callback when window destroyed */
 
-    g_signal_connect(G_OBJECT(ctk_window), "destroy",
-                     G_CALLBACK(ctk_window_destroyed), (gpointer) ctk_window);
+    g_signal_connect(G_OBJECT(ctk_window), "delete-event",
+                     G_CALLBACK(ctk_window_delete_event), (gpointer) ctk_window);
     
     return GTK_WIDGET(object);
 
@@ -1212,7 +1253,7 @@ static void add_page(GtkWidget *widget, GtkTextBuffer *help,
      */
 
     g_object_ref(G_OBJECT(widget));
-    gtk_object_sink(GTK_OBJECT(widget));
+    ctk_g_object_ref_sink(G_OBJECT(widget));
 
     gtk_tree_store_append(ctk_window->tree_store, child_iter, iter);
 
@@ -1252,24 +1293,23 @@ static GtkWidget *create_quit_dialog(CtkWindow *ctk_window)
     dialog = gtk_dialog_new_with_buttons("Really quit?",
                                          GTK_WINDOW(ctk_window),
                                          GTK_DIALOG_MODAL |
-                                         GTK_DIALOG_DESTROY_WITH_PARENT |
-                                         GTK_DIALOG_NO_SEPARATOR,
+                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_STOCK_CANCEL,
                                          GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_QUIT,
                                          GTK_RESPONSE_OK,
                                          NULL);
 
-    g_signal_connect(GTK_OBJECT(dialog), "response",
+    g_signal_connect(GTK_WIDGET(dialog), "response",
                      G_CALLBACK(quit_response),
-                     GTK_OBJECT(ctk_window));
+                     GTK_WIDGET(ctk_window));
 
     gtk_container_set_border_width(GTK_CONTAINER(dialog), 6);
     gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
     
     hbox = gtk_hbox_new(FALSE, 5);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 6);
-    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), hbox);
+    gtk_container_add(GTK_CONTAINER(ctk_dialog_get_content_area(GTK_DIALOG(dialog))), hbox);
     
     pixbuf = gtk_widget_render_icon(dialog, GTK_STOCK_DIALOG_QUESTION,
                                     GTK_ICON_SIZE_DIALOG, NULL);
@@ -1322,7 +1362,7 @@ static void quit_response(GtkWidget *button, gint response, gpointer user_data)
         save_settings_and_exit(ctk_window);
     }
 
-    gtk_widget_hide_all(ctk_window->quit_dialog);
+    gtk_widget_hide(ctk_window->quit_dialog);
 
 } /* quit_response() */
 
@@ -1370,7 +1410,7 @@ void add_special_config_file_attributes(CtkWindow *ctk_window)
  */
 
 static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
-                                NvCtrlAttributeHandle *gpu_handle,
+                                CtrlTarget *gpu_target,
                                 CtkEvent *ctk_event_gpu,
                                 GtkTextTagTable *tag_table,
                                 UpdateDisplaysData *data,
@@ -1378,25 +1418,35 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
 {
     GtkTextBuffer *help;
     ReturnStatus ret;
-    int *pData;
+    int *pData = NULL;
     int len;
     int i;
-    NvCtrlAttributeHandle *display_handle;
 
 
     /* retrieve the list of connected display devices */
 
-    ret = NvCtrlGetBinaryAttribute(gpu_handle, 0,
+    ret = NvCtrlGetBinaryAttribute(gpu_target, 0,
                                    NV_CTRL_BINARY_DATA_DISPLAYS_CONNECTED_TO_GPU,
                                    (unsigned char **)(&pData), &len);
     if ((ret != NvCtrlSuccess) || (pData[0] <= 0)) {
-        return;
+        goto done;
     }
 
     nvfree(data->display_iters);
+    nvfree(data->display_events);
+
     data->display_iters = calloc(pData[0], sizeof(GtkTreeIter));
+    if (!data->display_iters) {
+        goto done;
+    }
+
+    data->display_events = calloc(pData[0], sizeof(CtkEvent*));
+    if (!data->display_events) {
+        nvfree(data->display_iters);
+        goto done;
+    }
+
     data->num_displays = 0;
-    if (!data->display_iters) return;
 
 
     /*
@@ -1408,11 +1458,12 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
         int display_id = pData[i+1];
         char *logName;
         char *typeBaseName;
-        char *typeIdName;
+        char *randrName;
         GtkWidget *widget;
         gchar *title;
         CtkEvent *ctk_event;
-        CtrlHandles *handles = ctk_window->ctk_config->pCtrlHandles;
+        CtrlSystem *system = ctk_window->ctk_config->pCtrlSystem;
+        CtrlTarget *target;
 
         /*
          * Get the ctrl handle that was passed into ctk_main so that updated
@@ -1420,16 +1471,10 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
          * saved to the RC file when the UI is closed.
          */
 
-        display_handle = nv_get_target_handle(handles,
-                                              NV_CTRL_TARGET_TYPE_DISPLAY,
-                                              display_id);
-
-        if (!display_handle) {
-            display_handle =
-                nv_add_target(handles, NvCtrlGetDisplayPtr(gpu_handle),
-                              NV_CTRL_TARGET_TYPE_DISPLAY, display_id);
-
-            if (!display_handle) {
+        target = NvCtrlGetTarget(system, DISPLAY_TARGET, display_id);
+        if (!target) {
+            target = nv_add_target(system, DISPLAY_TARGET, display_id);
+            if (!target) {
                 continue;
             }
         }
@@ -1437,59 +1482,62 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
         /*
          * Rebuild Sub-systems of display handle
          */
-        NvCtrlAttributeRebuildSubsystems(display_handle,
-                                         NV_CTRL_ATTRIBUTES_ALL_SUBSYSTEMS);
-
-        ctk_event = CTK_EVENT(ctk_event_new(display_handle));
+        NvCtrlRebuildSubsystems(target, NV_CTRL_ATTRIBUTES_ALL_SUBSYSTEMS);
 
         /* Query display's names */
-        ret = NvCtrlGetStringAttribute(display_handle,
+        ret = NvCtrlGetStringAttribute(target,
                                        NV_CTRL_STRING_DISPLAY_NAME_TYPE_BASENAME,
                                        &typeBaseName);
         if (ret != NvCtrlSuccess) {
-            NvCtrlAttributeClose(display_handle);
             continue;
         }
-        ret = NvCtrlGetStringAttribute(display_handle,
+        ret = NvCtrlGetStringAttribute(target,
                                        NV_CTRL_STRING_DISPLAY_DEVICE_NAME,
                                        &logName);
         if (ret != NvCtrlSuccess) {
             logName = NULL;
         }
-        ret = NvCtrlGetStringAttribute(display_handle,
-                                       NV_CTRL_STRING_DISPLAY_NAME_TYPE_ID,
-                                       &typeIdName);
+        ret = NvCtrlGetStringAttribute(target,
+                                       NV_CTRL_STRING_DISPLAY_NAME_RANDR,
+                                       &randrName);
         if (ret != NvCtrlSuccess) {
-            typeIdName = NULL;
+            randrName = NULL;
         }
 
-        if (!logName && !typeIdName) {
+        if (!logName && !randrName) {
             title = g_strdup_printf("DPY-%d - (Unknown)", display_id);
         } else {
-            title = g_strdup_printf("%s - (%s)", typeIdName, logName);
+            title = g_strdup_printf("%s - (%s)", randrName, logName);
         }
-        XFree(logName);
-        XFree(typeIdName);
+        free(logName);
+        free(randrName);
 
         /* Create the page for the display */
-        widget = ctk_display_device_new(display_handle,
+        ctk_event = CTK_EVENT(ctk_event_new(target));
+        widget = ctk_display_device_new(target,
                                         ctk_window->ctk_config, ctk_event,
                                         ctk_event_gpu,
                                         title, typeBaseName, p);
-        if (widget == NULL) {
-            NvCtrlAttributeClose(display_handle);
-        } else {
+        if (widget != NULL) {
             help = ctk_display_device_create_help(tag_table,
                                                   CTK_DISPLAY_DEVICE(widget));
             add_page(widget, help, ctk_window, iter,
                      &(data->display_iters[data->num_displays]), title,
                      NULL, NULL, NULL);
+            data->display_events[data->num_displays] = ctk_event;
             data->num_displays++;
         }
+        else {
+            ctk_event_destroy(G_OBJECT(ctk_event));
+        }
 
-        XFree(typeBaseName);
+        free(typeBaseName);
         g_free(title);
     }
+
+ done:
+    free(pData);
+
 } /* add_display_devices() */
 
 
@@ -1530,13 +1578,14 @@ static void select_display_page(UpdateDisplaysData *data, gchar *name)
  *
  */
 
-static void update_display_devices(GtkObject *object, gpointer arg1,
+static void update_display_devices(GtkWidget *object,
+                                   CtrlEvent *event,
                                    gpointer user_data)
 {
     UpdateDisplaysData *data = (UpdateDisplaysData *) user_data;
 
     CtkWindow *ctk_window = data->window;
-    NvCtrlAttributeHandle *gpu_handle = data->gpu_handle;
+    CtrlTarget *gpu_target = data->gpu_target;
     GtkTreeIter parent_iter = data->parent_iter;
     GtkTextTagTable *tag_table = data->tag_table;
     GtkTreePath* parent_path;
@@ -1579,7 +1628,11 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
         gtk_tree_store_remove(ctk_window->tree_store, iter);
 
         /* unref the page so we don't leak memory */
-        g_object_unref(GTK_OBJECT(widget)); 
+        g_object_unref(G_OBJECT(widget));
+
+        /* Destroy the display CtkEvent */
+        ctk_event_destroy(G_OBJECT(
+                            data->display_events[data->num_displays - 1]));
 
         data->num_displays--;
     }
@@ -1589,7 +1642,7 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
     gtk_tree_model_get(GTK_TREE_MODEL(ctk_window->tree_store), &parent_iter,
                        CTK_WINDOW_WIDGET_COLUMN, &widget, -1);
 
-    add_display_devices(ctk_window, &parent_iter, gpu_handle,
+    add_display_devices(ctk_window, &parent_iter, gpu_target,
                         CTK_GPU(widget)->ctk_event,
                         tag_table, data, ctk_window->attribute_list);
 
